__author__ = "Felix Kleinert, Lukas Leufen"
__date__ = '2019-12-02'

from typing import Callable

import keras


def flatten_tail(input_X: keras.layers, name: str, bound_weight: bool = False, dropout_rate: float = 0.0,
                 window_lead_time: int = 4, activation: Callable = keras.activations.relu,
                 reduction_filter: int = 64, first_dense: int = 64):

    X_in = keras.layers.Conv2D(reduction_filter, (1, 1), padding='same', name='{}_Conv_1x1'.format(name))(input_X)

    X_in = activation(name='{}_conv_act'.format(name))(X_in)

    X_in = keras.layers.Flatten(name='{}'.format(name))(X_in)

    X_in = keras.layers.Dropout(dropout_rate, name='{}_Dropout_1'.format(name))(X_in)
    X_in = keras.layers.Dense(first_dense, kernel_regularizer=keras.regularizers.l2(0.01),
                              name='{}_Dense_1'.format(name))(X_in)
    if bound_weight:
        X_in = keras.layers.Activation('tanh')(X_in)
    else:
        try:
            X_in = activation(name='{}_act'.format(name))(X_in)
        except:
            X_in = activation()(X_in)

    X_in = keras.layers.Dropout(dropout_rate, name='{}_Dropout_2'.format(name))(X_in)
    out = keras.layers.Dense(window_lead_time, activation='linear', kernel_regularizer=keras.regularizers.l2(0.01),
                             name='{}_Dense_2'.format(name))(X_in)
    return out
